clear

% where are my custom functions?
addpath('../routines');
addpath('../bsp');


% ---   parameters for system size   ---
% parameter to generate system size n=2^k-1
k=menu('Problemgre, Testbeispiele',...
       ' 15',...
       ' 31',...
       ' 63',...
       '127',...
       '255')+4;
% system size
n=2^k-1;
fprintf('size of the problem %12.4e\n',n);
% --------------------------------------


% ---   parameters for multigrid cycle   ---
% number of levels, <=k
level=k-(0);
% ------------------------------------------




nmb=10;
strg=cell(nmb,1);
fun=cell(nmb,1);
strg{1}='laplace 1-D ';                                fun{1}='fun1';
strg{2}='laplace 1-D, alpha=1, beta=100';              fun{2}='fun2';
strg{3}='laplace 1-D, 1+sin(8pix)^2';                  fun{3}='fun3';
strg{4}='laplace 1-D, 1+sin(16pix)^2';                 fun{4}='fun4';
strg{5}='laplace 1-D, 1+sin(32pix)^2';                 fun{5}='fun5';
strg{6}='laplace 1-D, 1+e^{pix}sin(8pix)^2';           fun{6}='fun6';
strg{7}='laplace 1-D, 1+e^{2pix}sin(8pix)^2';          fun{7}='fun7';
strg{8}='laplace 1-D, 1+e^{8pix}sin(8pix)^2';          fun{8}='fun8';
strg{9}='laplace 1-D, jump. coeff [1,100,1e3,1e4]';    fun{9}='fun9';

strg{nmb}='END';



choice=menu('choice of sample problems',strg);...

if choice<nmb

   fprintf('%s\n',strg{choice});
  
   % depending on the problem generate initial matrix
   A=lap1d(n,fun{choice}); 
   n=size(A,1);
   
   % start flop counter
   flops(0);
   
   
   % solve linear system
   b=ones(n,1);
   
   % -------------------------   linear iteration   -------------------------
   fprintf('linear iteration\n');

   D=spdiags(diag(A),0,n,n);
   GS=tril(A);
   for i=1:2
       x=zeros(n,1);
       r=b;
       nrm=norm(r,inf);
       nrm_0=nrm;
       it=0;
       flops(0);
       while nrm>=sqrt(eps)*nrm_0
	     if i==1
	        % 1. Jacobi iteration
	        x=x+D\r;
	     else
	        % 2. Gauss-Seidel iteration
	        x=x+GS\r;
	     end % if-elseif-else
	     r=b-A*x;
	     nrm=norm(r,inf);
%             fprintf('%12.4e\n',nrm/nrm_0);
             it=it+1;
       end % while
       if i==1
	  fprintf('standard Diagonal iteration, #it: %4d, flops %12.4e\n',it,flops);
       else
	  fprintf('Gauss-Seidel      iteration, #it: %4d, flops %12.4e\n',it,flops);
       end % if-elseif-else
   end % for i
   % -----------------------   END linear iteration   -----------------------
   
end % if choice<3
