clear

% where are my custom functions?
addpath('../routines');
addpath('../bsp');


% ---   parameters for system size   ---
% parameter to generate system size n=2^k-1
k=menu('Problemgre, Testbeispiele',...
       '   49',...
       '  225',...
       '  961',...
       ' 3969',...
       '16129')+2;

% system size
N=2^k-1;
n=N*N;
fprintf('size of the problem %12.4e\n',n);
% --------------------------------------


% ---   parameters for multigrid cycle   ---
% number of levels, <=k
level=k-(0);
% ------------------------------------------




nmb=10;
strg=cell(nmb,1);
fun=cell(nmb,1);
strg{1}='laplace 2-D ';                                fun{1}='fun2_1';
strg{2}='laplace 2-D, alpha=1, beta=100';              fun{2}='fun2_2';
strg{3}='laplace 2-D, 1+sin(8pix)^2';                  fun{3}='fun2_3';
strg{4}='laplace 2-D, 1+sin(16pix)^2';                 fun{4}='fun2_4';
strg{5}='laplace 2-D, 1+sin(32pix)^2';                 fun{5}='fun2_5';
strg{6}='laplace 2-D, 1+e^{pix}sin(8pix)^2';           fun{6}='fun2_6';
strg{7}='laplace 2-D, 1+e^{2pix}sin(8pix)^2';          fun{7}='fun2_7';
strg{8}='laplace 2-D, 1+e^{8pix}sin(8pix)^2';          fun{8}='fun2_8';
strg{9}='laplace 2-D, jump. coeff [1,100,1e3,1e4]';    fun{9}='fun2_9';

strg{nmb}='END';



choice=menu('choice of sample problems',strg);...

if choice<nmb

   fprintf('%s\n',strg{choice});
  
   % depending on the problem generate initial matrix
   A=lap2d(N,fun{choice}); 
   n=size(A,1);
   
   % start flop counter
   flops(0);
   
   
   % solve linear system
   b=ones(n,1);
   
   % -------------------------   linear iteration   -------------------------
   fprintf('\nCG\n');

   for i=1:2
       % auxillary vector r. It contains the residual and after that the 
       % defect 
       % correction for x
       x=zeros(n,1);
       r=b-A*x;
       bs=size(b,2);
       flops(0);
   
       % ----------------------- CG iteration loop -----------------------
       % compute norm of the residual
       nrm=norm(r,inf);
       z=zeros(n,bs);
       p=zeros(n,bs);
       rho=0;
       rho_old=1;
       sigma=1;
       it=0;
       tol=nrm*sqrt(eps);
   
       D=spdiags(diag(A),0,n,n);
       GS=tril(A);
       
       while nrm>tol
      
	     it=it+1;
	     
	     % ------------ Solve preconditioned system: Mz=r ------------
	     if i==1
	        % 1. Jacobi preconditioning
		z=D\r;
	     else
		% 2. Gauss-Seidel preconditioning
		z=GS'\(D*(GS\r));
	     end % if-else
	     % --------- END Solve preconditioned system: Mz=r ----------
      
	     rho_old=rho;
	     % rho=r'*z
	     rho=r'*z;
	     if it==1
	        p=z;
%               tol=abs(rho)*sqrt(eps);
             else
	        beta=rho_old\rho;
		p=z+p*beta;
	     end % if-else it==1
      
	     z=A*p;
	     % sigma=p'*A*p
	     sigma=p'*z;
	     alpha=sigma\rho;
	     x=x+p*alpha;
	     r=r-z*alpha;
%            nrm=abs(rho);
             nrm=norm(r,inf);
             
       end % while nrm>tol
       % ------------------- END CG iteration loop -----------------------
       if i==1
	 fprintf('cg with diagonal preconditioning, #it: %4d, flops %12.4e\n',it,flops);
       else
	 fprintf('cg with sym. Gauss-Seidel prec.,  #it: %4d, flops %12.4e\n',it,flops);
       end % if-elseif-else
   end % for
   
end % if choice<3
