clear

% where are my custom functions?
addpath('../routines');
addpath('../bsp');


% ---   parameters for system size   ---
% parameter to generate system size n=2^k-1
k=menu('Problemgre, Testbeispiele',...
       '    63',...
       '   127',...
       '   255',...
       '   511',...
       '  1023',...
       '  2047',...
       '  4095',...
       ' 16192',...
       ' 32767')+5;
% system size
n=2^k-1;
% --------------------------------------


% ---   parameters for multigrid cycle   ---
% number of levels, <=k
level=k;-(k-5);
% ------------------------------------------




nmb=10;
strg=cell(nmb,1);
fun=cell(nmb,1);
strg{1}='laplace 1-D ';                                fun{1}='fun1';
strg{2}='laplace 1-D, alpha=1, beta=100';              fun{2}='fun2';
strg{3}='laplace 1-D, 1+sin(8pix)^2';                  fun{3}='fun3';
strg{4}='laplace 1-D, 1+sin(16pix)^2';                 fun{4}='fun4';
strg{5}='laplace 1-D, 1+sin(32pix)^2';                 fun{5}='fun5';
strg{6}='laplace 1-D, 1+e^{pix}sin(8pix)^2';           fun{6}='fun6';
strg{7}='laplace 1-D, 1+e^{2pix}sin(8pix)^2';          fun{7}='fun7';
strg{8}='laplace 1-D, 1+e^{8pix}sin(8pix)^2';          fun{8}='fun8';
strg{9}='laplace 1-D, jump. coeff [1,100,1e3,1e4]';    fun{9}='fun9';

strg{nmb}='END';



choice=menu('choice of sample problems',strg);...

if choice<nmb

   fprintf('%s\n',strg{choice});
   fprintf('size of the problem %12.4e\n',n);
   fprintf('number of levels: %d\n',level);

   if choice<=9
      IsSPD=-1;
   else
      IsSPD=0;
   end % if-else
   
   % list of matrices for any level
   Alist=cell(1,level);
   % projection (prolongation and restriction^T)
   Plist=cell(1,level-1);

   % depending on the problem generate initial matrix
   if choice==1
      Alist{1}=spdiags(ones(n,1)*[-1 2 -1],-1:1,n,n);
   else
      Alist{1}=lap1d(n,fun{choice}); 
   end % if
   % apply multigrid to the diagonal preconditioned system
   % in this case the averageing of the entries has to be adapted
   % D0=spdiags(1./sqrt(diag(Alist{1})),0,n,n);
   % Alist{1}=D0'*Alist{1}*D0;



   % --------------   Generation of the multilevel structure   --------------
   % dampening parameter for diagonal smoothing
   omegaD=2/3;
   % dampening parameter for improved Jacobi smoothing
   % omegaJ=2/3;
   nn=n;
   l=n+1;
   for m=1:level-1   
       % standard diagonal preconditioning
       Dlist{m}=omegaD*spdiags(1./diag(Alist{m}),0,nn,nn);
       GSFlist{m}=tril(Alist{m});
       GSBlist{m}=triu(Alist{m});
    
       % projection operator
       buff=spdiags(ones(nn,1)*[1/2 1 1/2],-1:1,nn,nn); 
       % D=spdiags(1./sqrt(diag(Alist{m})),0,nn,nn);
       % buff=speye(nn)-(1/2)*D'*Alist{m}*D; % with init. prec.
       % for ll=2:2:nn
       %     buff(:,ll)=buff(:,ll)/sum(buff(:,ll));
       % end % for
       Plist{m}=buff(:,2:2:nn);
       
    
       % improved Jacobi preconditioning
       % [L,U]=ImprovedJacobi(Alist{m},1,1);
       % Jlist{m}=omegaJ*L*U;
       
       l=l/2;
       nn=nn-l;
       
       % coarse grid system
       % Galerking-type
       Alist{m+1}=Plist{m}'*Alist{m}*Plist{m};
       % Geometric type
       % Alist{m+1}=lap1d(nn,fun{choice}); 

   end % for m
   if IsSPD
      Dlist{level}.U=chol(Alist{level});Dlist{level}.L=Dlist{level}.U';
   else
      [Dlist{level}.L,Dlist{level}.U]=lu(Alist{level});
   end % if-else
   GSFlist{level}.L=Dlist{level}.L;
   GSFlist{level}.U=Dlist{level}.U;
   GSBlist{level}.L=Dlist{level}.L;
   GSBlist{level}.U=Dlist{level}.U;
   % ------------   END Generation of the multilevel structure   ------------

   
   % call script file with iteration process
   iteration;
   
end % if choice<3
