function [P,C]=COARSERS(A,tol,coord)
% [P,C]=COARSERS(A,tol,coord)
%
% give a matrix A a rectangular coarse grid projection P matrix is
% constructed

n=size(A,1);

% tolerance for diagonal dominance
told=1-0.249;

if nargin==3
   M=[];
   clf;
   xmin=min(coord(:,1))-1; xmax=max(coord(:,1))+1;
   ymin=min(coord(:,2))-1; ymax=max(coord(:,2))+1;
   gplot(A,coord,'-r');hold on;
   axis([xmin xmax ymin ymax]);
   pause(.001)
end

% -----------------------   Set up strong connection   -----------------------
% weak connections (neighbours) for any node i
W=cell(1,n);
% strong connections (neighbours) for any node i
S=cell(1,n);
% nodes (neighbours) for which node i is a strong connection (neighbour)
ST=cell(1,n);
for j=1:n
    ST{j}=[];
end % for j
% strength (weight) of node i as strong neighbour od ST
lambda=zeros(1,n); 

% set of diagonal dominant nodes
Fdd=[];
% largest off-diagonal entry, row-wise
% mx=zeros(1,n);
for i=1:n
    % all negative off-diagonal entries
    J=setdiff(find(A(i,:)<0),i);
    % weak connections
    W{i}=J;
    
    d=abs(A(i,i));
    % strong connections of node i
    if ~isempty(J)
       rows=-sum(A(i,J));
       mx=max(-A(i,J));
       J=J(find(-A(i,J)>=tol*mx));
    else
       rows=0;
    end % if
    
    % strongly diagonal dominant rows don't have any strong connection
    if rows<=told*d
       S{i}=[];
       W{i}=[];
       Fdd=[Fdd i];
    else
       S{i}=J;
       W{i}=setdiff(W{i},J);
    end % if-else

    % adjoint strong connections of node i
    for j=S{i}
        ST{j}=[ST{j} i];
       	lambda(j)=lambda(j)+1; 
    end % for j   
end % for i


% sort lambda by size in increasing order
[lm,priority]=sort(lambda);
% set up double linked list
candidates=zeros(2,n);
candidates(:,priority(1))=[0;...             % successor(<=) in the list
                           priority(2)];     % predecessor(>=) in the list
for i=2:n-1
    candidates(:,priority(i))=[priority(i-1);...   % successor in the list
                               priority(i+1)];     % predecessor in the list
end % for i
candidates(:,priority(n))=[priority(n-1);...   % successor in the list
                           0];                 % predecessor in the list
ncandidates=n;
head=priority(n);
tail=priority(1);

% fprintf('lambda in decreasing order\n');
% j=head;
% for i=1:n
%     j,lambda(j)
%     j=candidates(1,j);
% end % for
% fprintf('lambda in increasing order\n');
% j=priority(1);
% for i=1:n
%     j,lambda(j)
%     j=candidates(2,j);
% end % for
% pause


% fprintf('lambda in decreasing order\n');
% jj=head;
% while jj>0
%       fprintf('(%3d,%12.4f)\n',jj,lambda(jj));
%       jj=candidates(1,jj);
% end % while


% first pass: Preliminary C-point choice
C=[]; 
F=[];
ndetermined=0;
% list of check marks. Let k=isdetermined(i). If k==0, then i has not
% been determined yet, neither as fine grid node nor as coarse grid node.
% If k<0, then i is number |k| in the list F of fine grid nodes. If k>0,
% then i is number k in the list of coarse grid nodes.
isdetermined=zeros(1,n);

% isolated (strong diagonal dom.) nodes belong to the fine grid
for j=Fdd
    F=[F j];
    isdetermined(j)=-length(F);
    ndetermined=ndetermined+1;
    % update list of candidates
    % is this the first element in the list?
    if candidates(2,j)==0
       % take the next smaller one as heading node
       head=candidates(1,j);
       candidates(:,j)=-1;      % this entry is no longer needed
       if head>0 % not the only remaining node
          candidates(2,head)=0; % so far there is no bigger lambda
       end
    % is this the last element in the list
    elseif candidates(1,j)==0
       % its predecessor will become the last element in the list
       tail=candidates(2,j);
       candidates(:,j)=-1;     % this entry is no longer needed
       candidates(1,tail)=0;   % so far there is no smaller lambda
    else
       succ=candidates(1,j);
       pred=candidates(2,j);
       candidates(:,j)=-1;      % this entry is no longer needed
       candidates(2,succ)=pred; 
       candidates(1,pred)=succ; 
    end % if-elseif-else
    if nargin==3
       gplot(1,coord(F(end),:),'or');hold on
       axis([xmin xmax ymin ymax]);
       pause(.001);
    end
end % for j

% fprintf('lambda in decreasing order\n');
% jj=head;
% while jj>0
%     fprintf('(%3d,%12.4f)\n',jj,lambda(jj));
%     jj=candidates(1,jj);
% end % while

while ndetermined<n
%       ndetermined
      % as coarse grid node we choose one that has maximal lambda
      i=head;
      % so far it successor will be come the new entry point in the list
      head=candidates(1,head);
      % update list of candidates
      candidates(:,i)=-1;     % this entry is no longer needed
      if head>0 % not the last entry
         candidates(2,head)=0;  % so far there is no bigger lambda
      end
% fprintf('coarse grid node: %d\n',i);
% fprintf('lambda in decreasing order\n');
% jj=head;
% while jj>0
%       fprintf('(%3d,%12.4f)\n',jj,lambda(jj)); 
%       jj=candidates(1,jj);
% end % while

      % new coarse grid node i
      C=[C i];
      isdetermined(i)=length(C);
      ndetermined=ndetermined+1;

      if nargin==3
         nn=size(M,1);
         if nn>0
            mm=abs(A(:,C(1:nn)))'*(abs(A)*abs(A(:,C(end))));
         else
            mm=sparse(0,1);
         end
         MM=[speye(nn)                mm;...
               mm'      abs(A(:,C(end)))'*abs(A)*abs(A(:,C(end)))];
         gplot(MM,coord(C,:),'og');hold on;gplot(MM,coord(C,:),'-g');
         MM(1:end-1,1:end-1)=M;
         M=MM;
         axis([xmin xmax ymin ymax]);
         pause(.001);
      end
      
      % each node for which i is a strong connection will be moved to the set
      % of fine grid nodes
      for j=ST{i}
          % node is still available
          if isdetermined(j)==0
% fprintf('fine grid node: %d\n',j);
             % move it to the fine grid
             F=[F j];
             if nargin==3
                gplot(1,coord(F(end),:),'or');hold on
                axis([xmin xmax ymin ymax]);
                pause(.001);
             end
             isdetermined(j)=-length(F);
             ndetermined=ndetermined+1;
             % update list of candidates
             % is this the first element in the list?
             if candidates(2,j)==0
                % take the next smaller one as heading node
                head=candidates(1,j);
                candidates(:,j)=-1;      % this entry is no longer needed
                if head>0 % not the only remaining node
                   candidates(2,head)=0; % so far there is no bigger lambda
                end
             % is this the last element in the list?
             elseif candidates(1,j)==0
                % its predecessor will become the last element in the list
                tail=candidates(2,j);
                candidates(:,j)=-1;     % this entry is no longer needed
                candidates(1,tail)=0;   % so far there is no smaller lambda
             else
                succ=candidates(1,j);
                pred=candidates(2,j);
                candidates(:,j)=-1;      % this entry is no longer needed
                candidates(2,succ)=pred; 
                candidates(1,pred)=succ; 
             end % if-elseif-else
             
             for l=S{j}
                 % node is still available
                 if isdetermined(l)==0
% fprintf('fine grid neighbour: %d\n',l);
                    lambda(l)=lambda(l)+1; 
                    % update list of candidates. 
                    % remove l from the double linked list and correct
                    % l's predecessor and l's successor 
                    if head~=l
                       p=candidates(2,l);
                       s=candidates(1,l);
                       if p>0
                          candidates(1,p)=s;
                       end
                       if s>0
                          candidates(2,s)=p;
                       else
                          tail=p;
                       end % if-else
                       
                       % search downward in double linked list until a
                       % smaller lambda has been found
                       flag=-1;
                       succ=head;
                       while flag
                             % is this (still) the last element in the list?
                             if succ==0
                                tail=l;
                                p=candidates(2,l);
                                candidates(1,p)=l;
                                flag=0;
                             elseif lambda(l)<lambda(succ)
                                succ=candidates(1,succ);
                             else % lambda(l)>=lambda(succ)
                                % l has to be inserted before succ
                                p=candidates(2,succ);
                                candidates(2,succ)=l;
                                candidates(:,l)=[succ;...
                                                 p];
                                if p>0
                                   candidates(1,p)=l;
                                else % l become the first entry in the list
                                   head=l;
                                end
                                flag=0;
                             end % if-elseif-else
                       end % while flag
                    end % if head~=l
% fprintf('lambda in decreasing order\n');
% jj=head;           
% while jj>0         
%       fprintf('(%3d,%12.4f)\n',jj,lambda(jj));
%       jj=candidates(1,jj);
% end % while        
                 end % if isdetermined(l)==0
             end % for l
          end % if  isdetermined(j)==0
      end % for j
      
      for j=setdiff(S{i},ST{i})
          % node is still available
          if isdetermined(j)==0
% fprintf('coarse grid neighbour: %d\n',j);
             lambda(j)=lambda(j)-1;            
             % update list of candidates. 
             % remove j from the double linked list and correct
             % j's predecessor and j's successor 
             if tail~=j
                p=candidates(2,j);
                s=candidates(1,j);
                if p>0
                   candidates(1,p)=s;
                else
                   head=s;
                end % if-else
                if s>0
                   candidates(2,s)=p;
                end
                   
                % search upward in double linked list until a bigger lambda
                % has been found
                flag=-1;
                pred=tail;
                while flag
                      % is this (still) the first element in the list?
                      if pred==0
                         head=j;
                         s=candidates(1,j);
                         candidates(2,s)=j;
                         flag=0;
                      elseif lambda(j)>lambda(pred)
                         pred=candidates(2,pred);
                      else % lambda(j)<=lambda(pred)
                         % j has to be inserted after pred
                         s=candidates(1,pred);
                         candidates(1,pred)=j;
                         candidates(:,j)=[s;...
                                          pred];
                         if s>0
                            candidates(2,s)=j;
                         else % j becomes the last entry in the list
                            tail=j;
                         end
                         flag=0;
                      end % if-elseif-else
                end % while
             end % if tail~=j
% fprintf('lambda in decreasing order\n');
% jj=head;    
% while jj>0      
%       fprintf('(%3d,%12.4f)\n',jj,lambda(jj));
%       jj=candidates(1,jj);
% end % while   
          end % if isdetermined(j)==0
      end % for j
% figure(1)
% lambda,C,lambda(C),F,lambda(F)
% plot(1:n,lambda,'b+',C,lambda(C),'mo',F,lambda(F),'g*'),
% pause(.01)
% figure(3)
end % while

% fprintf('lambda in decreasing order\n');
% jj=head;
% while jj>0
%       fprintf('(%3d,%12.4f)\n',jj,lambda(jj));
%       jj=candidates(1,jj);
% end % while



% This is only done for convenience and routines like Gauss-Seidel
% C=sort(C);





% Second pass: add further nodes to the coarse grid if necessary
P=spalloc(n,n,nnz(A));
% diagonal dominant nodes don't need to be tested
cnt=length(Fdd)+1;
while cnt<=length(F)
      % check fine grid point
      i=F(cnt);
% fprintf('testing F-point %d\n',i);
      % determine strong neighbours of i that are in the coarse grid
      Ci=[];
      tCi=0;
      Dsi=[];
      for k=S{i}
          % is this a coarse grid node?
          if isdetermined(k)>0
             Ci=[Ci k];
          else % this is a strong F-F connection. This should be avoided!
             Dsi=[Dsi k];
          end
      end % for k
      % diagonal compensation for the weak neighbours 
      Pii=A(i,i)+sum(A(i,W{i}));
      % we are lucky, no strong F-F connections
      if isempty(Dsi)
         % define final weights
         P(i,Ci)=-Pii\A(i,Ci);
         cnt=cnt+1;
      else % we are unlucky, there are strong F-F connections
% fprintf('strong connections\n');
% for j=1:length(Dsi)
%     fprintf('%3d',Dsi(j));
% end          
% fprintf('\n');  
	      % check if these F-F connection are at least connected to Ci 
         cntj=1;
         while cntj<=length(Dsi)
               j=Dsi(cntj);
               % do i and j share common coarse grid points
               if tCi~=0
                  CiCj=intersect(S{j},union(Ci,tCi));
               else
                  CiCj=intersect(S{j},Ci);
               end % if-else
% [j,CiCj]     
	            if ~isempty(CiCj)
                  % we have to do a local elimination, keep j in Dsi and move
                  % to the next strong connection
                  cntj=cntj+1;
              else % no, then j will be temporarily moved to the coarse grid
                  % is this already the second node?
                  if tCi~=0
% fprintf('At least two serious strong connections. %d becomes C-point\n',i);
                     % In this case it is cheaper to move i to the coarse grid
                     % than to include two of i's neighbours to the coarse grid
                     % no strong neighbours anymore
                     Dsi=[];
                     % remove i from the set of F-points by overwriting it with
                     % the last element in the list
                     F(cnt)=F(end);
                     isdetermined(F(end))=-cnt;
                     F(end)=[];
                     % add i to the set of C-points
                     C=[C i];
                     isdetermined(i)=length(C);
                     tCi=0;
                     if nargin==3
                        nn=size(M,1);
                        mm=abs(A(:,C(1:nn)))'*(abs(A)*abs(A(:,C(end))));
                        MM=[speye(nn)                mm;...
                              mm'      abs(A(:,C(end)))'*abs(A)*abs(A(:,C(end)))];
                        gplot(MM,coord(C,:),'og');hold on;gplot(MM,coord(C,:),'-g');
                        MM(1:end-1,1:end-1)=M;
                        M=MM;
                        axis([xmin xmax ymin ymax]);
                        pause(.001);
                     end
                  else % j becomes a coarse grid node, temporarily
                     tCi=j;
% fprintf('serious strong connection %d detected\n',j);		     
                     % It might be that there a no serious strong connections
                     % anymore. But clearly j is the first serious strong
                     % connection, because otherwise tCi=0. 
                     % j has to be removed from the list strong connections
                     Dsi(cntj)=Dsi(end);
                     Dsi(end)=[];
                  end % if-else
               end % if-else
         end % while cntj
         
         % if i has not been moved to the coarse grid in the mean time         
         if isdetermined(i)<0
            % in this case we have to construct an interpolation
            if tCi~=0
               Ci=union(Ci,tCi);
            end            
            P(i,Ci)=-A(i,Ci);
         end % if
         
         % There are still strong connections of i, but they will be handled
         % by a local elimination
         if ~isempty(Dsi)
            for j=Dsi
                distweight=-sum(A(j,Ci));
                P(i,Ci)=P(i,Ci)+A(j,Ci)*(A(i,j)/distweight);
            end
         end % if
         
         % The problem of serious F-F connection has been solved by moving
         % one strong connection temporarily to the caorse grid. In this case
         % the node now becomes a permanent member of the coarse grid
         if tCi~=0
            % remove tCi from the list of F-points
            % if this F-point occured before i in the list, then we ignore it
            % This gives an inconsistent list of F-points but we don't need it
            % anymore. But if tCi appears later in the F-point list then it
            % will be overwritten by the finel element in the list, because 
            % these F-nodes still have to be checked
            pos=-isdetermined(tCi);
            if pos>cnt
               F(pos)=F(end);
               isdetermined(F(end))=-pos;
               F(end)=[];
            end % if
            % Now add tCi to the coarse grid
            C=[C tCi];
            isdetermined(tCi)=length(C);
            P(tCi,:)=sparse(1,n);
            if nargin==3
               nn=size(M,1);
               mm=abs(A(:,C(1:nn)))'*(abs(A)*abs(A(:,C(end))));
               MM=[speye(nn)                mm;...
                     mm'      abs(A(:,C(end)))'*abs(A)*abs(A(:,C(end)))];
               gplot(MM,coord(C,:),'og');hold on;gplot(MM,coord(C,:),'-g');
               MM(1:end-1,1:end-1)=M;
               M=MM;
               axis([xmin xmax ymin ymax]);
               pause(.001);
            end
         end % if
         
         
         % if i has not been moved to the coarse grid in the mean time
         if isdetermined(i)<0
            % define final weights
            P(i,Ci)=Pii\P(i,Ci);
            cnt=cnt+1;
            % otherwise F(cnt) has been overwritten
         end
      end % if isempty(Dsi)
      
end % while (cnt<=...)


% final coarse grid matrix
% for coarse grid nodes we use identical interpolation
for i=C
    P(i,i)=1;
end % for i
% P(C,C)=speye(length(C));
% reduce matrix to F-C connections
P=P(:,C);
